import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * My own random number generator, with bigger choice.
 */
final public class MyRandom extends Random {

    final static double ROOT2PI = Math.sqrt( 2 * Math.PI );
    
    	/**
	 * this method generates a symmetric alpha stable random
	 * variable with mean 0 and skewness 0. The parameter alpha
	 * is trusted to be in the range (0, 2]
	 */
    public double nextSymmetricAlpha(double alpha) {
	double V = Math.PI * (nextDouble() - 0.5);
	double k = alpha * V;
	double W = - Math.log(nextDouble())/Math.cos(V-k);
	double h = W * Math.cos(V);
	double s = Math.abs(Math.sin(k) * Math.exp( - Math.log(h)/alpha ) * W);
	s = ( nextDouble() < 0.5 ) ? -s : s;
	return s;
    }

	/**
	 * this method generates a symmetric one dimensional Student t
	 * distribution with parameter nu > 0.
	 */
    public double nextStudent(int nu) {

	double t = 0;
	for(int i = 0; i < nu; i++ ) {
	    double s = nextGaussian();
	    t += s * s;
	}
	t = nextGaussian() / Math.sqrt(t/nu);
	return t;
    }
    
	/**
	 * This method returns a random variable  whose density is equal to
	 * the supremum of all normal densities with variance sigma squared
	 * and mean belonging to the interval (-e, e). In other words, the
	 * density is flat inside the interval (-e,e) and gaussian outside.
	 */
    public double nextPlateauGaussian(double sigma, double e) {
	if( nextDouble() > 2 * e / ( ROOT2PI * sigma + 2 * e ) ) {
	    double w = nextGaussian() * sigma;
	    return ( w > 0 ) ? w + e : w - e;
	}
	else {
	    return 2 * e * nextDouble() - e;
	}
    }

    // lambda is the mean
    public double nextExponential(double lambda) {
	return( -Math.log(nextDouble())*lambda);
    }

    // Weibull with 
    // invalpha = 1/alpha where alpha is shape parameter
    // beta = scale parameter
    // gamma = treshold parameter
    // CDF is given by 
    // prob( X <= x ) = 1 - exp(-(x - gamma)/ beta)^alpha ) if x > gamma
    //                = 0                                   if x <= gamma
    public double nextWeibull(double invalpha, double beta, double gamma) {
	return gamma + beta * Math.pow(-Math.log(nextDouble()), invalpha);
    }

    // generates a Gamma random variable with shape parameter a and scale =1. Note
    // that if X is such a gamma random variable, then cX is Gamma(a,c).  
    public double nextGamma(double a) {

	boolean small = false;

	if( a < 1.0 ) {
	    small = true;
	    a += 1.0;
	}

	double x,y;
	double c = Math.sqrt(2*a - 1);
	
	a -= 1.0;
	do {
	    y = Math.tan(Math.PI * nextDouble());
	    x = c * y + a;
	} while( x <= 0 || 
		 nextDouble() > (1 + y*y) * 
		 Math.exp(a * Math.log(x/a) - c * y) );

	if( small ) 
	    x *= Math.pow(nextDouble(), 1.0/a);

	return x;	    
    }
}




